use crate::ems::pcs::PcsCsvData;
use crate::internal::modbus::{BaseModbusCode, ModbusDataType};
use std::collections::HashMap;

#[derive(Debug)]
pub struct ModbusCode {
    pub addr: Option<u16>,
    pub code: Option<u8>,
    pub data: Option<[u8; 2]>,
    pub name: Option<String>,
    pub data_type: Option<ModbusDataType>,
    pub factor: Option<f32>,
    pub unit: Option<String>,
    pub note: Option<HashMap<i32, String>>,
    pub desc: Option<String>,
}

impl ModbusCode {}


impl BaseModbusCode for ModbusCode {
    fn addr(&self) -> Option<u16> {
        self.addr
    }
}

impl TryFrom<PcsCsvData> for ModbusCode {
    type Error = anyhow::Error;

    fn try_from(value: PcsCsvData) -> Result<Self, Self::Error> {
        let addr = value.address;
        let code = value.code;
        let name = value.name;
        let mut data_type = None;
        if let Some(ts) = value.t {
            data_type = if ts == "bool" {
                Some(ModbusDataType::BOOL)
            } else if ts == "u16" {
                Some(ModbusDataType::U16)
            } else if ts == "s16" {
                Some(ModbusDataType::S16)
            } else {
                None
            }
        };
        let factor = value.factor;
        let unit = value.unit;
        let desc = value.desc;
        let mut note = None;
        if let Some(n) = value.note {
            let mut hash_map = HashMap::new();
            for x in n.split('\n').into_iter() {
                let kv = x.split(':').collect::<Vec<&str>>();
                if let Some(key) = kv.get(0) {
                    let k = if key.starts_with("0x") {
                        i32::from_str_radix(key.trim().replace("0x", "").as_ref(), 16)?
                    } else {
                        key.parse::<i32>()?
                    };
                    if let Some(val) = kv.get(1) {
                        hash_map.insert(k, val.trim().to_string());
                    }
                }
            }
            note = Some(hash_map);
        }
        Ok(ModbusCode {
            addr,
            code,
            data: None,
            name,
            data_type,
            factor,
            unit,
            note,
            desc,
        })
    }
}
